package com.hz.pm.api.projectdeclared.utils;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjUtil;
import lombok.Builder;
import lombok.Data;

import java.io.Serializable;
import java.util.*;

/**
 * <p>
 * BeanDiffUtil
 * </p>
 *
 * @author WendyYang
 * @since 10:27 2024/8/6
 */
public class BeanDiffUtil {

    private BeanDiffUtil() {
    }

    @Data
    @Builder
    public static class FieldDiff implements Serializable {

        private static final long serialVersionUID = -9003132377177378460L;

        private String fieldName;

        private String fileType;

        private Object oldValue;

        private Object newValue;

    }

    /**
     * 比较两个对象的不同属性值，包括嵌套的对象属性。
     *
     * @param obj1 第一个对象
     * @param obj2 第二个对象
     * @return 返回一个 Map，其中 key 是不同属性的名称，value 是一个包含两个值的数组，第一个值来自 obj1，第二个值来自 obj2
     */
    private static List<FieldDiff> diff(Object obj1, Object obj2, boolean diffSameField, String... ignoreFields) {
        Map<String, Object> map1 = BeanUtil.beanToMap(obj1);
        Map<String, Object> map2 = BeanUtil.beanToMap(obj2);
        if (ArrayUtil.isNotEmpty(ignoreFields)) {
            for (String ignoreField : ignoreFields) {
                map2.remove(ignoreField);
                map1.remove(ignoreField);
            }
        }
        // 创建结果 Map
        List<FieldDiff> result = new ArrayList<>();

        // 获取两个 Map 的 keySet
        Set<String> keys1 = map1.keySet();
        Set<String> keys2 = map2.keySet();

        // 计算不同的属性
        Collection<String> diffKeys = CollUtil.disjunction(keys1, keys2);

        if (!diffSameField) {
            // 输出不同的属性及其值
            for (String key : diffKeys) {
                Object oldValue = map1.getOrDefault(key, null);
                Object newValue = map2.getOrDefault(key, null);
                result.add(FieldDiff.builder()
                        .fieldName(key)
                        .fileType((oldValue == null ? newValue.getClass() : oldValue.getClass()).getTypeName())
                        .oldValue(oldValue)
                        .newValue(newValue)
                        .build());
            }

        }

        // 输出相同的属性，但是值不同
        Collection<String> commonKeys = CollUtil.intersection(keys1, keys2);
        for (String key : commonKeys) {
            Object value1 = map1.get(key);
            Object value2 = map2.get(key);
            if (value1 == null && value2 == null) {
                continue;
            }
            Class<?> valueClass = value1 == null ? value2.getClass() : value1.getClass();
            // 如果是嵌套对象，则递归比较
            if (!BeanUtil.isBean(valueClass) && !ObjUtil.equals(value1, value2)) {
                Object oldValue = map1.getOrDefault(key, null);
                Object newValue = map2.getOrDefault(key, null);
                result.add(FieldDiff.builder()
                        .fieldName(key)
                        .fileType((oldValue == null ? newValue.getClass() : oldValue.getClass()).getTypeName())
                        .oldValue(oldValue)
                        .newValue(newValue)
                        .build());
            }
        }

        return result;
    }

    /**
     * 比较两个对象的不同属性值，包括嵌套的对象属性。
     *
     * @param obj1 第一个对象
     * @param obj2 第二个对象
     * @return 返回一个 Map，其中 key 是不同属性的名称，value 是一个包含两个值的数组，第一个值来自 obj1，第二个值来自 obj2
     */
    public static List<FieldDiff> diffSameField(Object obj1, Object obj2, String... ignoreFields) {
        return diff(obj1, obj2, true, ignoreFields);
    }

    /**
     * 比较两个对象的不同属性值，包括嵌套的对象属性。
     *
     * @param obj1 第一个对象
     * @param obj2 第二个对象
     * @return 返回一个 Map，其中 key 是不同属性的名称，value 是一个包含两个值的数组，第一个值来自 obj1，第二个值来自 obj2
     */
    public static List<FieldDiff> diff(Object obj1, Object obj2, String... ignoreFields) {
        return diff(obj1, obj2, false, ignoreFields);
    }
}
